//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this source code is subject to the terms of the Microsoft end-user
// license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
// If you did not accept the terms of the EULA, you are not authorized to use
// this source code. For a copy of the EULA, please see the LICENSE.RTF on your
// install media.
//
/***********************************************************************

    MODULE:
    CeDialer.c

    ABSTRACT:
    This code sample shows how to initialize an application's use
    of TAPI, open a line device, negotiate an API version to use,
    translate an address into another format, place a call on an
    opened line device, close an opened line device, and shut down
    an application's use of the line abstraction of the API.

    FUNCTIONS:
    ConnectUsingProc - Processes messages sent to IDD_CONNECTUSING dialog.
    CurrentLineClose - Closes the opened line device.
    DialerDlgProc    - Processes messages sent to IDD_CEDIALER dialog.
    DialingProc      - Processes messages sent to IDD_DIALING dialog.
    GetLineInfo      - Get line information for selected line.
    InitAddrCB       - Fills address list box on IDD_CONNECTUSING dialog.
    InitApplication  - Initializes application.
    InitializeTAPI   - Initializes the application's use of the Tapi.dll.
    InitInstance     - Instance initialization.
    InitLineCB       - Fills line list box on IDD_CONNECTUSING dialog.
    lineCallbackFunc - Monitors line and call status.
    MainWndProc      - Processes messages sent to the main window.
    MakeCanonicalNum - Converts phone number to canonical address format.
    MakePhoneCall    - Demonstrates the use of TAPI call functions.
    WinMain          - Application entry point and initialization.

***********************************************************************/

// Include the proper version of TAPI
#define ERR_NONE              0
#define TAPI_VERSION_1_0      0x00010003
#define TAPI_VERSION_1_4      0x00010004
#define TAPI_VERSION_2_0      0x00020000
#define TAPI_CURRENT_VERSION  TAPI_VERSION_2_0

#define REASONABLE_BUFFER_SIZE  512

#include <windows.h>
#include <aygshell.h>
#include <tapi.h>
#include <strsafe.h>
#include "resource.h"
#include "CeDialer.h"
#include "extapi.h"






	/*Initialize the CeDialer class*/

	CeDialer::CeDialer(HWND hwnd){
		m_hwnd = hwnd;


	}

/* Set up a telephone connection */
	int CeDialer::Start(LPCTSTR lpszPhoneNum){
		DWORD ini_ret;

		if(ini_ret=InitializeTAPI()){


		}
		if (ini_ret == ERR_NONE)
		MakePhoneCall (lpszPhoneNum);

		return 1;

		}

/*Hangs up the current telephone connection */
		int CeDialer::Stop(){

			CurrentLineClose ();


			return 1;
		}

	/*Set up log functionality */

		void CeDialer::setVerbosity(int v){
			//NULL;
		}

		/* Sets the microphone active*/
		void CeDialer::set_micOn(){
			micOn();


		}
		/* Sets the microphone passive*/
		void CeDialer::set_micOff(){
			micOff();
		}




HINSTANCE g_hInst_    = NULL;    // hInstance of the application
HWND g_hwndMain      = NULL;    // Handle to the main window
HWND g_hwndDial      = NULL;    // Handle to the dialing window
HWND g_hwndDialerDlg = NULL;    // Handle to the dialer dialog
DWORD hDevice;

TCHAR g_szTitle[] = TEXT("CeDialer TAPI Sample");
// CeDialer application window name
TCHAR g_szAppName[] = TEXT("CeDialer");
// Main window class name

HLINEAPP g_hLineApp = NULL;     // Application's use handle for TAPI
// (lineInitialize)
HCALL g_hCall = NULL;           // Handle to the open line device on
// which the call is to be originated
// (lineMakeCall)
LONG g_MakeCallRequestID = 0;   // Request identifier returned by
// lineMakeCall
LONG g_DropCallRequestID = 0;   // Request identifier returned by
// lineDroph
BOOL g_bCurrentLineAvail = TRUE;// Indicates line availability


TCHAR g_szCurrentNum[TAPIMAXDESTADDRESSSIZE + 1] ;


// Current phone number
TCHAR g_szLastNum[TAPIMAXDESTADDRESSSIZE + 1];
// Last called phone number


DWORD g_dwNumDevs = 0;          // Number of line devices available
DWORD g_dwCurrentLineID = -1;   // Current line device identifier
DWORD g_dwCurrentLineAddr = -1; // Current line address

LINEINFO g_CurrentLineInfo;     // Contains the current line information
LINEINFO *g_lpLineInfo = NULL;  // Array that contains all the lines'
// information

#define ErrorBox(_s) MessageBox(g_hwndDialerDlg,_s, \
TEXT("CeDialer Message"),MB_OK)
#define MENU_HEIGHT 26          // Height of menu bar at screen bottom


// Purpose: Determine at runtime if the app is running on a smartphone device
static BOOL IsSmartphone()
{
    TCHAR tszPlatform[64];

    if (TRUE == SystemParametersInfo(SPI_GETPLATFORMTYPE,
        sizeof(tszPlatform)/sizeof(*tszPlatform),tszPlatform,0))
    {
        if (0 == _tcsicmp(TEXT("Smartphone"), tszPlatform))
        {
            return TRUE;
        }
    }
    return FALSE;
}


/***********************************************************************

FUNCTION:
lineCallbackFunc

PURPOSE:
This is a callback function invoked to determine status and events on
the line device, addresses, or calls.

***********************************************************************/
VOID CALLBACK lineCallbackFunc (
    DWORD hDevice,
    DWORD dwMsg,
    DWORD dwCallbackInstance,
    DWORD dwParam1,
    DWORD dwParam2,
    DWORD dwParam3
    )
{

    BOOL bCloseLine = FALSE;
    LPTSTR lpszStatus;
    lpszStatus = TEXT(" ");

    switch (dwMsg)
          {
          case LINE_CALLSTATE:  // Sent after change of call state

              // dwParam1 is the specific CALLSTATE change that is occurring.
              switch (dwParam1)
              {
              case LINECALLSTATE_DIALTONE:
                  lpszStatus = TEXT("Dial tone");
                  break;

              case LINECALLSTATE_DIALING:
                  lpszStatus = TEXT("Dialing...");
                  break;

              case LINECALLSTATE_PROCEEDING:
                  lpszStatus = TEXT("Dialing completed, call proceeding.");
                  break;

              case LINECALLSTATE_RINGBACK:
                  lpszStatus = TEXT("Ring back");
                  break;

              case LINECALLSTATE_CONNECTED:
                  lpszStatus = TEXT("Connected");
                  break;

              case LINECALLSTATE_BUSY:
                  lpszStatus = TEXT("Line busy, shutting down.");
                  bCloseLine = TRUE;
                  break;

              case LINECALLSTATE_IDLE:
                  lpszStatus = TEXT("Line is idle");
                  break;

              case LINECALLSTATE_SPECIALINFO:
                  lpszStatus =TEXT("Special Information, couldn't dial number");
                  bCloseLine = TRUE;
                  break;

              case LINECALLSTATE_DISCONNECTED:
                  {
                      LPTSTR lpszDisconnected;
                      lpszDisconnected = TEXT(" ");

                      switch (dwParam2)
                      {
                      case LINEDISCONNECTMODE_NORMAL:
                          lpszDisconnected = TEXT("Remote party disconnected");
                          break;

                      case LINEDISCONNECTMODE_UNKNOWN:
                          lpszDisconnected = TEXT("Disconnected: Unknown reason");
                          break;

                      case LINEDISCONNECTMODE_REJECT:
                          lpszDisconnected = TEXT("Remote Party rejected call");
                          break;

                      case LINEDISCONNECTMODE_PICKUP:
                          lpszDisconnected =
                              TEXT("Disconnected: Local phone picked up");
                          break;

                      case LINEDISCONNECTMODE_FORWARDED:
                          lpszDisconnected = TEXT("Disconnected: Forwarded");
                          break;

                      case LINEDISCONNECTMODE_BUSY:
                          lpszDisconnected = TEXT("Disconnected: Busy");
                          break;

                      case LINEDISCONNECTMODE_NOANSWER:
                          lpszDisconnected = TEXT("Disconnected: No Answer");
                          break;

                      case LINEDISCONNECTMODE_BADADDRESS:
                          lpszDisconnected = TEXT("Disconnected: Bad address");
                          break;

                      case LINEDISCONNECTMODE_UNREACHABLE:
                          lpszDisconnected = TEXT("Disconnected: Unreachable");
                          break;

                      case LINEDISCONNECTMODE_CONGESTION:
                          lpszDisconnected = TEXT("Disconnected: Congestion");
                          break;

                      case LINEDISCONNECTMODE_INCOMPATIBLE:
                          lpszDisconnected = TEXT("Disconnected: Incompatible");
                          break;

                      case LINEDISCONNECTMODE_UNAVAIL:
                          lpszDisconnected = TEXT("Disconnected: Unavailable");
                          break;

                      case LINEDISCONNECTMODE_NODIALTONE:
                          lpszDisconnected = TEXT("Disconnected: No dial tone");
                          break;

                      default:
                          lpszDisconnected = TEXT("Disconnected: Unknown reason");
                          break;
                      } // end switch (dwParam2)

                      bCloseLine = TRUE;
                      lpszStatus = lpszDisconnected;
                      break;
                  } // end case LINECALLSTATE_DISCONNECTED:
      }   // end switch (dwParam1)


      if (bCloseLine)
      {
          CurrentLineClose ();
          if (g_hwndDial)
              SendMessage (g_hwndDial, WM_COMMAND, MAKEWPARAM(IDOK,0), 0);
      }
      break;

    case LINE_LINEDEVSTATE:

        switch (dwParam1)
        {
        case LINEDEVSTATE_RINGING:
            lpszStatus = TEXT("Ringing");
            break;

        case LINEDEVSTATE_OUTOFSERVICE:
            lpszStatus = TEXT("The line selected is out of service.");

            CurrentLineClose ();
            if (g_hwndDial)
                SendMessage (g_hwndDial, WM_COMMAND, MAKEWPARAM(IDOK,0), 0);
            break;

        case LINEDEVSTATE_DISCONNECTED:
            lpszStatus = TEXT("The line selected is disconnected.");

            CurrentLineClose ();
            if (g_hwndDial)
                SendMessage (g_hwndDial, WM_COMMAND, MAKEWPARAM(IDOK,0), 0);
            break;

        case LINEDEVSTATE_MAINTENANCE:
            lpszStatus = TEXT("The line selected is out for maintenance.");

            CurrentLineClose ();
            if (g_hwndDial)
                SendMessage (g_hwndDial, WM_COMMAND, MAKEWPARAM(IDOK,0), 0);
            break;

        case LINEDEVSTATE_TRANSLATECHANGE:
            break;

        case LINEDEVSTATE_REMOVED:
            ErrorBox (
                TEXT("The Line device has been removed; no action taken."));
            break;

        case LINEDEVSTATE_REINIT:
            {
                // This usually means that a service provider has changed in
                // such a way that requires TAPI to REINIT. Note that there
                // are both soft REINITs and hard REINITs. Soft REINITs do not
                // require a full shutdown but an informational change that
                // historically required a REINIT to force the application to
                // deal with.  TAPI API Version 1.3 applications require a
                // full REINIT for both hard and soft REINITs.

                switch(dwParam2)
                {
                    // This is the hard REINIT. TAPI is waiting for everyone to
                    // shut down. Our response is to immediately shut down any
                    // calls, shut down our use of TAPI and notify the user.
                case 0:
                    if (MessageBox (
                        g_hwndMain,
                        TEXT("Tapi line configuration has been changed. ")
                        TEXT("You have to shut down CeDialer to\n")
                        TEXT("re-initialize the use of tapi.dll. Do ")
                        TEXT("you want to shut down CeDialer now?"),
                        TEXT("Warning"),
                        MB_YESNO) == IDYES)
                    {
                        lineShutdown (g_hLineApp);
                        DestroyWindow (g_hwndMain);
                    }

                    break;

                case LINE_CREATE:
                    lineCallbackFunc (hDevice, dwParam2, dwCallbackInstance,
                        dwParam3, 0, 0);
                    break;

                case LINE_LINEDEVSTATE:
                    lineCallbackFunc (hDevice, dwParam2, dwCallbackInstance,
                        dwParam3, 0, 0);
                    break;

                    // There might be other reasons to send a soft REINIT.
                    // No need to shut down for these reasons.
                default:
                    break;
                }
            }

        default:
            break;
        }
        break;

        case LINE_REPLY:

            // Reply from the lineMakeCall function.
            if ((LONG)dwParam1 == g_MakeCallRequestID)
            {
                // If an error occurred on making the call.
                if (dwParam2 != ERR_NONE)
                {
                    lpszStatus = TEXT("Closing line");
                    if (dwParam2 == LINEERR_CALLUNAVAIL)
                        lpszStatus = TEXT("The line is not available.");

                  /*  if (g_hwndDial)
                        SendDlgItemMessage(g_hwndDial, IDC_STATUSMESSAGE,
                        WM_SETTEXT, 0,(LPARAM)lpszStatus);
                */    CurrentLineClose ();

                    if (g_hwndDial)
                        SendMessage (g_hwndDial, WM_COMMAND, MAKEWPARAM(IDOK,0), 0);
                }
            }

            break;

        case LINE_CREATE:

            // dwParam1 is the device identifier of the new line.
            if (dwParam1 >= g_dwNumDevs)
            {
                DWORD dwLineID;
                LINEINFO *lpLineInfo;

                g_dwNumDevs = dwParam1 + 1;

                // Allocate a buffer for storing LINEINFO for all the lines.
                if (!(lpLineInfo = (LPLINEINFO) LocalAlloc (
                    LPTR,
                    sizeof (LINEINFO) * g_dwNumDevs)))
                {
                    break;
                }

                // Assume we just add a new line, the lines are sequential and
                // the new line is the last one.
                for (dwLineID = 0; dwLineID < dwParam1; ++dwLineID)
                {
                    lpLineInfo[dwLineID] = g_lpLineInfo[dwLineID];
                }

                // Get the new line information.
                GetLineInfo (dwParam1, &lpLineInfo[dwParam1]);

                LocalFree (g_lpLineInfo);
                g_lpLineInfo = lpLineInfo;
            }
            break;

        case LINE_CLOSE:
            if (g_CurrentLineInfo.hLine == (HLINE) hDevice)
            {
                lpszStatus = TEXT("Closing line");
/*                if (g_hwndDial)
                    SendDlgItemMessage(g_hwndDial, IDC_STATUSMESSAGE,
                    WM_SETTEXT, 0,(LPARAM)lpszStatus);*/
                CurrentLineClose ();

                if (g_hwndDial)
                    SendMessage (g_hwndDial, WM_COMMAND, MAKEWPARAM(IDOK,0), 0);
            }
            break;

        case LINE_ADDRESSSTATE:
        case LINE_CALLINFO:
        case LINE_DEVSPECIFIC:
        case LINE_DEVSPECIFICFEATURE:
        case LINE_GATHERDIGITS:
        case LINE_GENERATE:
        case LINE_MONITORDIGITS:
        case LINE_MONITORMEDIA:
        case LINE_MONITORTONE:
        case LINE_REMOVE:
        case LINE_REQUEST:
        default:
            break;
  }

}

/***********************************************************************

FUNCTION:
CurrentLineClose

PURPOSE:
This function closes the opened line device.

***********************************************************************/
VOID CurrentLineClose ()
{
    Sleep(2000); // Time delay to provide message display.

    // If lineMakeCall succeeded, then drop the call.
    if (g_hCall)
    {
        g_DropCallRequestID = lineDrop (g_hCall, NULL, 0);
        Sleep(5000);
        lineDeallocateCall (g_hCall); // Deallocate call handle.
    }

    // Close the current line.
    if (g_CurrentLineInfo.hLine)
        lineClose (g_CurrentLineInfo.hLine);

    // Reinitialize the variables.
    g_CurrentLineInfo.hLine = NULL;
    g_bCurrentLineAvail = TRUE;
    g_hCall = NULL;
}


/***********************************************************************

FUNCTION:
GetLineInfo

PURPOSE:
Get line information for selected line.

***********************************************************************/


DWORD GetLineInfo(
    DWORD dwLineID,
    LPLINEINFO lpLineInfo
    )
{
    DWORD dwSize;
    DWORD     dwReturn;
    LPTSTR lpszLineName = NULL;
    LPLINEDEVCAPS lpLineDevCaps = NULL;

    // Negotiate the API version number. If it fails, return to dwReturn.
    if (dwReturn = lineNegotiateAPIVersion (
        g_hLineApp,                   // TAPI registration handle
        dwLineID,                     // Line device to be queried
        TAPI_VERSION_1_0,             // Least recent API version
        TAPI_CURRENT_VERSION,         // Most recent API version
        &(lpLineInfo->dwAPIVersion),  // Negotiated API version
        NULL))                        // Must be NULL; the provider-
        // specific extension is not
        // supported on Windows CE
    {
        goto exit;
    }

    dwSize = sizeof (LINEDEVCAPS);

    // Allocate enough memory for lpLineDevCaps.
    do
    {
        if (!(lpLineDevCaps = (LPLINEDEVCAPS) LocalAlloc (LPTR, dwSize)))
        {
            dwReturn = LINEERR_NOMEM;
            goto exit;
        }

        lpLineDevCaps->dwTotalSize = dwSize;

        if (dwReturn = lineGetDevCaps (g_hLineApp,
            dwLineID,
            lpLineInfo->dwAPIVersion,
            0,
            lpLineDevCaps))
        {
            goto exit;
        }

        // Stop if the allocated memory is equal to or greater than the
        // needed memory.
        if (lpLineDevCaps->dwNeededSize <= lpLineDevCaps->dwTotalSize)
            break;

        dwSize = lpLineDevCaps->dwNeededSize;
        LocalFree (lpLineDevCaps);
        lpLineDevCaps = NULL;

    } while (TRUE);

    // Store the line information in *lpLineInfo.
    lpLineInfo->dwPermanentLineID = lpLineDevCaps->dwPermanentLineID;
    lpLineInfo->dwNumOfAddress = lpLineDevCaps->dwNumAddresses;
    lpLineInfo->bVoiceLine =
        (lpLineDevCaps->dwMediaModes & LINEMEDIAMODE_INTERACTIVEVOICE);

    // Allocate memory for lpszLineName.
    if (!(lpszLineName = (LPTSTR) LocalAlloc (LPTR, REASONABLE_BUFFER_SIZE)))
    {
        dwReturn = LINEERR_NOMEM;
        goto exit;
    }

    // Store the line name in *lpszLineName.
    if (lpLineDevCaps->dwLineNameSize > 0)
    {
        StringCbCopy(
            lpszLineName,
            REASONABLE_BUFFER_SIZE,
            (LPTSTR)((LPSTR)lpLineDevCaps + lpLineDevCaps->dwLineNameOffset));


		}
    else
    {
        _stprintf (lpszLineName, TEXT("Line %d"), dwLineID);
    }

    // Copy lpszLineName to lpLineInfo->lpszLineName.
    lstrcpy (lpLineInfo->szLineName, lpszLineName);

    dwReturn = ERR_NONE;

exit:

    if (lpLineDevCaps)
        LocalFree (lpLineDevCaps);

    if (lpszLineName)
        LocalFree (lpszLineName);

    return dwReturn;
}


/***********************************************************************

FUNCTION:
InitializeTAPI

PURPOSE:
Initialize the application's use of Tapi.dll.

***********************************************************************/
DWORD InitializeTAPI ()
{
    DWORD dwLineID,
        dwReturn,
        dwTimeCount = GetTickCount ();

	LPTSTR lpszLineName = NULL;
	LPLINEDEVCAPS lpLineDevCaps = NULL;

    TCHAR szWarning[] = TEXT("Cannot initialize tapi.dll.")
        TEXT("\nQuit all other telephony")
        TEXT("\nprograms, and try again.");


    // Initialize the application's use of Tapi.dll. Keep trying until the
    // user cancels or stops getting LINEERR_REINIT.
    while ( (dwReturn = lineInitialize (&g_hLineApp,
        g_hInst_,
        (LINECALLBACK) lineCallbackFunc,
        g_szAppName,
        &g_dwNumDevs)) == LINEERR_REINIT)
    {
        // Bring up the message box if 5 seconds have passed.
        if (GetTickCount () > 5000 + dwTimeCount)
        {
            if (MessageBox (g_hwndMain, szWarning, TEXT("Warning"),
                MB_OKCANCEL) == IDOK)
                break;

            // Reset the time counter.
            dwTimeCount = GetTickCount ();
        }
    }

    // If function "lineInitialize" fails, then return.
    if (dwReturn)
        return dwReturn;

    // If there is no device, then return.
    if (g_dwNumDevs == 0)
    {
        ErrorBox (TEXT("There are no line devices available."));
        return LINEERR_NODEVICE;
    }

    // Allocate buffer for storing LINEINFO for all the available lines.
    if (! (g_lpLineInfo = (LPLINEINFO) LocalAlloc (
        LPTR,
        sizeof (LINEINFO) * g_dwNumDevs)))
    {
        return LINEERR_NOMEM;
    }

	//MessageBox(NULL,L"Kommer den til initialize",NULL,MB_OK);
    // Fill lpLineInfo[] for every line.
    for (dwLineID = 0; dwLineID < g_dwNumDevs; ++dwLineID)
    {

        GetLineInfo (dwLineID, &g_lpLineInfo [dwLineID]);

		if(wcscmp(g_lpLineInfo[dwLineID].szLineName, L"Cellular Line") == 0){
		//MessageBox(NULL, L"Cellular ", NULL, MB_OK);
		g_dwCurrentLineID = dwLineID;
		g_CurrentLineInfo = g_lpLineInfo [g_dwCurrentLineID];



		}


    }

    return ERR_NONE;
}


/***********************************************************************

FUNCTION:
MakePhoneCall

PURPOSE:
Demonstrates the use of lineOpen, lineTranslateAddress, lineMakeCall.

***********************************************************************/
VOID MakePhoneCall (LPCTSTR lpszPhoneNum)
{
    DWORD dwReturn,
    dwSizeOfTransOut = sizeof (LINETRANSLATEOUTPUT),
    dwSizeOfCallParams = sizeof (LINECALLPARAMS);

    LPLINECALLPARAMS lpCallParams = NULL;
    LPLINETRANSLATEOUTPUT lpTransOutput = NULL;

    WCHAR szDialablePhoneNum[80 + 1] = {'\0'};//TAPIMAXDESTADDRESSSIZE
    int err = 0;

    // Initialize g_MakeCallRequestID.
    g_MakeCallRequestID = 0;



    // Open the current line.

    if (dwReturn = lineOpen (
        g_hLineApp,                 // Usage handle for TAPI
        g_dwCurrentLineID,          // Cannot use the LINEMAPPER value
        &g_CurrentLineInfo.hLine,   // Line handle
        g_CurrentLineInfo.dwAPIVersion,
        // API version number
        0,                          // Must set to zero for Windows CE
        0,                          // No data passed back
        LINECALLPRIVILEGE_NONE,     // Can only make an outgoing call
        0,                          // Media mode
        NULL))                      // Must set to NULL for Windows CE
    {


        //goto exit;
    }

    // Call translate address before dialing.
    do
    {
        // Allocate memory for lpTransOutput.
        if (!(lpTransOutput = (LPLINETRANSLATEOUTPUT) LocalAlloc (
            LPTR,
            dwSizeOfTransOut)))
        {
            goto exit;
        }

        lpTransOutput->dwTotalSize = dwSizeOfTransOut;

        if (dwReturn = lineTranslateAddress (
            g_hLineApp,               // Usage handle for TAPI
            g_dwCurrentLineID,        // Line device identifier
            g_CurrentLineInfo.dwAPIVersion,
            // Highest TAPI version supported
            lpszPhoneNum,             // Address to be translated
            0,                        // Must be 0 for Windows CE
            0,                        // No associated operations
            lpTransOutput))           // Result of the address translation
        {
            goto exit;
        }

        if (lpTransOutput->dwNeededSize <= lpTransOutput->dwTotalSize)
            break;
        else
        {
            dwSizeOfTransOut = lpTransOutput->dwNeededSize;
            LocalFree (lpTransOutput);
            lpTransOutput = NULL;
        }

    } while (TRUE);

    dwSizeOfCallParams += lpTransOutput->dwDisplayableStringSize;

    if (!(lpCallParams = (LPLINECALLPARAMS) LocalAlloc (
        LPTR,
        dwSizeOfCallParams)))
    {
        goto exit;
    }

    ZeroMemory(lpCallParams, dwSizeOfCallParams);

    // Set the call parameters.

	//MessageBox(NULL,L"fr params",NULL,MB_OK);
	g_dwCurrentLineAddr = 0;
    lpCallParams->dwTotalSize      = dwSizeOfCallParams;
    lpCallParams->dwBearerMode     = LINEBEARERMODE_VOICE;
    lpCallParams->dwMediaMode      = LINEMEDIAMODE_INTERACTIVEVOICE; //changed from LINEMEDIAMODE_DATAMODEM
    lpCallParams->dwCallParamFlags = LINECALLPARAMFLAGS_IDLE;
    lpCallParams->dwAddressMode    = LINEADDRESSMODE_ADDRESSID;
    lpCallParams->dwAddressID      = g_dwCurrentLineAddr;
    lpCallParams->dwDisplayableAddressSize =
        lpTransOutput->dwDisplayableStringSize;
    lpCallParams->dwDisplayableAddressOffset = sizeof (LINECALLPARAMS);





    // Save the translated phone number for dialing.
    lstrcpy (szDialablePhoneNum,
        (LPTSTR) ((LPBYTE) lpTransOutput +
        lpTransOutput->dwDialableStringOffset));
    memcpy((LPBYTE) lpCallParams + lpCallParams->dwDisplayableAddressOffset,
        (LPBYTE) lpTransOutput + lpTransOutput->dwDisplayableStringOffset,
        lpTransOutput->dwDisplayableStringSize);

    // Make the phone call. lpCallParams should be NULL if the default
    // call setup parameters are requested.


    g_MakeCallRequestID = lineMakeCall (g_CurrentLineInfo.hLine,
        &g_hCall,
        szDialablePhoneNum, //szDialablePhoneNum,
        0,
        lpCallParams);   //lpcallparams

	//if (dwParam2 == LINEERR_CALLUNAVAIL)
      //                  lpszStatus = TEXT("The line is not available.");

    if (g_MakeCallRequestID > 0)
    {
        g_bCurrentLineAvail = FALSE;

    }
    else
    {
        ErrorBox (TEXT("Failed in making the call, ")
            TEXT("\nfunction lineMakeCall failed."));
        CurrentLineClose ();
    }

exit :

    if (lpCallParams)
        LocalFree (lpCallParams);

    if (lpTransOutput)
        LocalFree (lpTransOutput);

    // If the make call did not succeed but the line was opened,
    // then close it.
    if ((g_MakeCallRequestID <= 0) && (g_CurrentLineInfo.hLine))
        CurrentLineClose ();

    return;
}


int micOn(){
	 lineSetMuteState(
		g_CurrentLineInfo.hLine,
		LINEMUTESTATE_MUTEDISABLED);

	 return 1;
}


int micOff(){

 lineSetMuteState(
		g_CurrentLineInfo.hLine,
		LINEMUTESTATE_MUTEENABLED);

 return 1;
}

/***********************************************************************

  FUNCTION:
  ConnectUsingProc

    PURPOSE:
    Processes messages sent to IDD_CONNECTUSING dialog box.

***********************************************************************/
BOOL CALLBACK ConnectUsingProc(
    HWND hwnd,
    UINT uMsg,
    WPARAM wParam,
    LPARAM lParam
    )
{
    SHINITDLGINFO shidi;
    SHMENUBARINFO mbi;

    switch (uMsg)
    {
    case WM_INITDIALOG:
        {
            // Create OK button in navigation bar and size dialog.
            shidi.dwMask = SHIDIM_FLAGS;
            shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIZEDLG;
            shidi.hDlg =  hwnd;
            SHInitDialog(&shidi);


            if (IsSmartphone())
            {

                memset(&mbi, 0, sizeof(SHMENUBARINFO));
                mbi.cbSize     = sizeof(SHMENUBARINFO);
                mbi.hwndParent = hwnd;
               // mbi.nToolBarId = IDR_OKMENU;
                mbi.hInstRes   = g_hInst_;
                mbi.nBmpId     = 0;
                mbi.cBmpImages = 0;

                if (SHCreateMenuBar(&mbi) == 0)
                    return FALSE;
            }


            return TRUE;
        }


    }
    return FALSE;
}


// END CEDIALER.C

