// blocksock.cpp (CBlockingSocketError, CBlockingSocket, CHttpBlockingSocket)
#include "blocksock.h"
#include "resource.h"


// Class CBlockingSocketError

HWND hwndMain; // Global window handle
HMENU hMenu; // Global menu handle



void CBlockingSocket::Cleanup()
{
	if(m_hSocket == NULL) return;

	//VERIFY(closesocket(m_hSocket) != SOCKET_ERROR);
	_ASSERTE(closesocket(m_hSocket) != SOCKET_ERROR);
    
	if(closesocket(m_hSocket) == SOCKET_ERROR)
		::MessageBox(hwndMain,L"Error: Can't close socket",NULL,MB_OK);
	m_hSocket = NULL;
}

void CBlockingSocket::Create(int nType  /*= SOCK_STREAM */)
{
	_ASSERTE(m_hSocket == NULL);

#ifndef _WIN32_WCE 
	WORD wVersionRequested;
	WSADATA wsaData;
	int err;

	wVersionRequested = MAKEWORD( 2, 2 );

	err = WSAStartup( wVersionRequested, &wsaData );
	if ( err != 0 ) {
		// Tell the user that we could not find a usable 
		// WinSock DLL.                                  
		SocketError(L"Can't find a usable WinSock DLL");
		exit(0);
	}
#endif

	
	if((m_hSocket = socket(AF_INET, nType, 0)) == INVALID_SOCKET) {
		SocketError(L"Create");
		exit(0);
	}

	// Create a non_blocking socket.

	/*unsigned long blockmode = 1;
	
	int error;

	error = ioctlsocket(m_hSocket,FIONBIO,(unsigned long *)&blockmode);

	if(r == SOCKET_ERROR) {
		 SocketError(L"Can't create a non-blocking socket");
	}*/

}

void CBlockingSocket::Bind(LPCSOCKADDR psa)
{
	_ASSERTE(m_hSocket != NULL);
	if(bind(m_hSocket, psa, sizeof(SOCKADDR)) == SOCKET_ERROR) {
		 SocketError(L"Bind");
		 exit(0);
	}
}

void CBlockingSocket::Listen()
{
	_ASSERTE(m_hSocket != NULL);
	if(listen(m_hSocket, 5) == SOCKET_ERROR) {
		 SocketError(L"Listen");
		 exit(0);
	}
}

BOOL CBlockingSocket::Accept(CBlockingSocket& sConnect, LPSOCKADDR psa)
{
	_ASSERTE(m_hSocket != NULL);
	_ASSERTE(sConnect.m_hSocket == NULL);
	int nLengthAddr = sizeof(SOCKADDR);
	sConnect.m_hSocket = accept(m_hSocket, psa, &nLengthAddr);
	if(sConnect == INVALID_SOCKET) {
		// no exception if the listen was canceled
		if(WSAGetLastError() != WSAEINTR) {
			 SocketError(L"Accept");
		}
		return FALSE;
	}
	return TRUE;
}

void CBlockingSocket::Close()
{
	_ASSERTE(m_hSocket != NULL);
	
	if(closesocket(m_hSocket) == SOCKET_ERROR) {
		// should be OK to close if closed already
		 SocketError(L"Close");
	}
	
	m_hSocket = NULL;
}

int CBlockingSocket::Connect(LPCSOCKADDR psa)
{
	_ASSERTE(m_hSocket != NULL);
	// should timeout by itself
	if(connect(m_hSocket, psa, sizeof(SOCKADDR)) == SOCKET_ERROR) {
		 SocketError(L"Connect");
		 return SOCKET_ERROR;
	}
	return 0;
}

int CBlockingSocket::Write(const char* pch, const int nSize, const int nSecs)
{
	int nBytesSent = 0;
	int nBytesThisTime;
	const char* pch1 = pch;
	do {
		nBytesThisTime = Send(pch1, nSize - nBytesSent, nSecs);
		nBytesSent += nBytesThisTime;
		pch1 += nBytesThisTime;
	} while(nBytesSent < nSize);

	return nBytesSent;
}

int CBlockingSocket::Send(const char* pch, const int nSize, const int nSecs)
{
	
	_ASSERTE(m_hSocket != NULL);
	// returned value will be less than nSize if client cancels the reading
	FD_SET fd = {1, m_hSocket};
	TIMEVAL tv = {nSecs, 0};
	if(select(0, NULL, &fd, NULL, &tv) == 0) {
		 SocketError(L"Send timeout");
		 exit(0);
	}
	int nBytesSent = 0;
	
	
	if((nBytesSent = send(m_hSocket, pch, nSize, 0)) == SOCKET_ERROR) {
		 SocketError(L"Send");
		 exit(0);
	}
	
	return nBytesSent;
}

int CBlockingSocket::Receive(char* pch, const int nSize, const int nSecs)
{
	_ASSERTE(m_hSocket != NULL);
	FD_SET fd = {1, m_hSocket};
	TIMEVAL tv = {nSecs, 0};
	
	if(select(0, &fd, NULL, NULL, &tv) == 0) {
		SocketWarning(L"Receive timeout");
	}
	int nBytesReceived = 0;
	
	unsigned long bytesOnSocket = 0;
	
	int error;

	// Check if there is any data on the socket.

	error = ioctlsocket(m_hSocket,FIONREAD,&bytesOnSocket);

	if(error == SOCKET_ERROR) {
		 SocketError(L"Can't check the socket for data to read");
		 exit(0);
	}
   
	// Will not use the recv function if there is no data on the socket.
	// The reason for this is to avoid blocking of the socket.
	
	if(bytesOnSocket > 0) {
		// Read data from the socket
		if((nBytesReceived = recv(m_hSocket,pch, nSize, 0)) == SOCKET_ERROR) {
			 SocketError(L"Receive");
			 exit(0);
		}
	}
	
	return nBytesReceived;
}





/*
 * Function: ReadNbytes(void *buf, int n, int wait)
 *           Try to read n bytes and only n from socket to buf
 *           within wait microseconds. 
 *           wait = number of microseconds to wait for data
 *                   0: immediate return
 *                  -1: wait forever
 *
 * Return value: >=0: number of chars transferred to buf
 *               < 0: an error occurred
 *               // READ_EOF: EOF detected 
 *               // (READ_EOF #defined in mysock.h)
 */

int CBlockingSocket::ReadNbytes(void *buf, int n, int wait) {
                                 
  int      bcount,br;
  char    *Ptr;
  bcount = 0;
  br     = 0;
  Ptr    = (char *)buf;

  _ASSERTE(m_hSocket != NULL);
  // FD_SET fd = {1, m_hSocket};

  while (bcount < n) { // Loop until full buffer

	struct timeval tval, *t;  // time structure for select() 
	fd_set readfds;           // for select() 
	int nread = 0;            // number of char read 
	if (wait >= 0) {
	  tval.tv_sec = wait / 1000000;
	  tval.tv_usec = wait % 1000000;
	  t = &tval;
	} else {
	  t = NULL;
	}
	FD_ZERO(&readfds);
	FD_SET(m_hSocket, &readfds);
	if ((select(m_hSocket + 1, 
				&readfds, NULL, NULL, t) > 0) && (FD_ISSET(m_hSocket, &readfds))) {
	  if ((br = recv(m_hSocket, (char *)Ptr, n-bcount, 0))  > 0) {
		bcount += br;
		Ptr    += br;
	  }
	}
	
	// There is no data, or EOF, anyway, we signal nothing to read
	if (br == 0) {
		break;
	}
  }

  return bcount;
}











int CBlockingSocket::ReceiveDatagram(char* pch, const int nSize, LPSOCKADDR psa, const int nSecs)
{
	_ASSERTE(m_hSocket != NULL);
	FD_SET fd = {1, m_hSocket};
	TIMEVAL tv = {nSecs, 0};
	if(select(0, &fd, NULL, NULL, &tv) == 0) {
		SocketWarning(L"Receive timeout");
	}

	// input buffer should be big enough for the entire datagram
	int nFromSize = sizeof(SOCKADDR);
	int nBytesReceived = recvfrom(m_hSocket, pch, nSize, 0, psa, &nFromSize);
	if(nBytesReceived == SOCKET_ERROR) {
		 SocketError(L"ReceiveDatagram");
		 exit(0);
	}
	return nBytesReceived;
}

int CBlockingSocket::SendDatagram(const char* pch, const int nSize, LPCSOCKADDR psa, const int nSecs)
{
	_ASSERTE(m_hSocket != NULL);
	FD_SET fd = {1, m_hSocket};
	TIMEVAL tv = {nSecs, 0};
	if(select(0, NULL, &fd, NULL, &tv) == 0) {
		 SocketError(L"Send timeout");
		 exit(0);
	}

	int nBytesSent = sendto(m_hSocket, pch, nSize, 0, psa, sizeof(SOCKADDR));
	if(nBytesSent == SOCKET_ERROR) {
		 SocketError(L"SendDatagram");
		 exit(0);
	}
	return nBytesSent;
}

void CBlockingSocket::GetPeerAddr(LPSOCKADDR psa)
{
	_ASSERTE(m_hSocket != NULL);
	// gets the address of the socket at the other end
	int nLengthAddr = sizeof(SOCKADDR);
	if(getpeername(m_hSocket, psa, &nLengthAddr) == SOCKET_ERROR) {
		 SocketError(L"GetPeerName");
		 exit(0);
	}
}

void CBlockingSocket::GetSockAddr(LPSOCKADDR psa)
{
	_ASSERTE(m_hSocket != NULL);
	// gets the address of the socket at this end
	int nLengthAddr = sizeof(SOCKADDR);
	if(getsockname(m_hSocket, psa, &nLengthAddr) == SOCKET_ERROR) {
		 SocketError(L"GetSockName");
	}
}

//static
CSockAddr CBlockingSocket::GetHostByName(const char* pchName, const USHORT ushPort /* = 0 */)
{
	hostent* pHostEnt = gethostbyname(pchName);
	if(pHostEnt == NULL) {
		 SocketError(L"GetHostByName");
	}
	ULONG* pulAddr=NULL;
	if(pHostEnt) pulAddr = (ULONG*) pHostEnt->h_addr_list[0];
	SOCKADDR_IN sockTemp;
	sockTemp.sin_family = AF_INET;
	sockTemp.sin_port = htons(ushPort);
	if(pHostEnt) sockTemp.sin_addr.s_addr = *pulAddr; // address is already in network byte order
	return sockTemp;
}

//static
const char* CBlockingSocket::GetHostByAddr(LPCSOCKADDR psa)
{
	hostent* pHostEnt = gethostbyaddr((char*) &((LPSOCKADDR_IN) psa)->sin_addr.s_addr, 4, PF_INET);
	if(pHostEnt == NULL) {
		 SocketError(L"GetHostByAddr");
	}
	return pHostEnt->h_name; // caller shouldn't delete this memory
}

// CSockAddr CBlockingSocket::GetHostByAddrHostent(LPCSOCKADDR psa, const USHORT ushPort)
CSockAddr CBlockingSocket::MUST_GetHostByAddr(char *IPadr, 
											  const USHORT ushPort)
{
	
	hostent*    host;
	struct      sockaddr_in sa;
	char        Hostname[256];
	SOCKADDR_IN sockTemp;
	u_long      IPadress;
	
	switch (IPadress = ToIPAdr(IPadr)) {
	case 0:
		if ((host = gethostbyname(IPadr))) {
			strcpy(Hostname,host->h_name);
			memcpy (&sa.sin_addr,host->h_addr, host->h_length); 
		}
		else {
			
		}
		break;
	default:
		
		sa.sin_addr.s_addr = IPadress;		  		  
		if ((host = gethostbyaddr((char *)&(sa.sin_addr), 
			sizeof(sa.sin_addr), 
			AF_INET))) {
			memcpy (host->h_addr, (char *)&sa.sin_addr, host->h_length);
			sa.sin_family = host->h_addrtype;
			sa.sin_port = htons((u_short)ushPort);
		}
		else { 	
		}
		
	}
	
	ULONG* pulAddr = NULL;
	if(host == NULL) {
		
		
		sockTemp.sin_family      = AF_INET;
		sockTemp.sin_port        = htons(NULL);
		sockTemp.sin_addr.s_addr = NULL;	

		// jek
		// SocketError(L"MUST_GetHostByAddr");
		
		// jek	- try it anyway - we ignore if no name servers available on domain
		char tmp[1024];
		strcpy(tmp,IPadr);
		int cnt   = 0;
		char *Addr[32+1];
		int index = 0;
		Addr[index++] = &tmp[0];
		while (tmp[cnt] != '\0') {
			if (tmp[cnt] == '.') {
				tmp[cnt]      = 0;
				Addr[index++] = &tmp[cnt+1];
			}
			cnt++;
		}  
		int addr0_i,addr1_i,addr2_i,addr3_i;
		sscanf(Addr[0], "%i", &addr0_i);
		sscanf(Addr[1], "%i", &addr1_i); 
		sscanf(Addr[2], "%i", &addr2_i);
		sscanf(Addr[3], "%i", &addr3_i);
		tmp[0] = (char)addr0_i;
		tmp[1] = (char)addr1_i;
		tmp[2] = (char)addr2_i;
		tmp[3] = (char)addr3_i;
		
		pulAddr                  = (ULONG*) tmp;
		sockTemp.sin_family      = AF_INET;
		sockTemp.sin_port        = htons(ushPort);
		sockTemp.sin_addr.s_addr = *pulAddr; // address is already in network byte order
		
		
	}
	else {
		if(host) pulAddr = (ULONG*) host->h_addr_list[0];
		sockTemp.sin_family = AF_INET;
		sockTemp.sin_port = htons(ushPort);
		if(host) sockTemp.sin_addr.s_addr = *pulAddr; // address is already in network byte order
	}
	return sockTemp;
}


/*
 * Static Function: ToIPAdr(char *host)
 *                  convert string "n1.n2.n3.n4" to IP-adress
 *                  Stricter than inet_addr(): returns zero if
 *                  there are not four groups of decimal integers
 *                  of value 0 - 255, separated by '.' in the string
 *
 * Return value: IP-adress, or 0 if format error
 */

u_long CBlockingSocket::ToIPAdr(char *hname) {

  char c, *p, hostname[32];
  long ipadr = 0;
  int i, tmp;

  if (strlen(hname) > 31) return(0);

  p = strcpy(hostname, hname);


  while((c = *p++) != '\0') {
	if ((c != '.') && ( (c < '0') || (c > '9') ) ) return(0);
  }

  p = strtok(hostname, ".");
  for(i = 0; i < 4; i++) {
    if (p == NULL) return(0);
    if (((tmp = atoi(p)) > 255) || (tmp < 0)) return(0);
    ipadr = (ipadr << 8) + tmp;
    p = strtok(NULL, ".");
  }
  return htonl(ipadr);
}





// Class CHttpBlockingSocket


CHtmlBlockingSocket::CHtmlBlockingSocket()
{
	m_pReadBuf = new char[nSizeRecv];
	m_nReadBuf = 0;
	
	// jek
	socketStatus = 0;
}

CHtmlBlockingSocket::~CHtmlBlockingSocket()
{
	delete [] m_pReadBuf;
}

int CHtmlBlockingSocket::ReadLine(char* pch, const int nSize, const int nSecs)
// reads an entire header line through CRLF (or socket close)
// inserts zero string terminator, object maintains a buffer
{
	int nBytesThisTime = m_nReadBuf;
	int nLineLength = 0;
	char* pch1 = m_pReadBuf;
	char* pch2;
	do {
		// look for lf (assume preceded by cr)
		if((pch2 = (char*) memchr(pch1 , '\n', nBytesThisTime)) != NULL) {
			_ASSERTE((pch2) > m_pReadBuf);
			nLineLength = (pch2 - m_pReadBuf) + 1;
			if(nLineLength >= nSize) nLineLength = nSize - 1;
			memcpy(pch, m_pReadBuf, nLineLength); // copy the line to caller
			m_nReadBuf -= nLineLength;
			memmove(m_pReadBuf, pch2 + 1, m_nReadBuf); // shift remaining characters left
			break;
		}
		pch1 += nBytesThisTime;
		nBytesThisTime = Receive(m_pReadBuf + m_nReadBuf, nSizeRecv - m_nReadBuf, nSecs);
		//if(nBytesThisTime <= 0) { // sender closed socket or line longer than buffer
		if(nBytesThisTime < 0) {
			SocketError(L"ReadHeaderLine");
			exit(0);
			//// break;
		}
		m_nReadBuf += nBytesThisTime;
	}
	while(TRUE);
	*(pch + nLineLength) = '\0';
	return nLineLength;
}

int CHtmlBlockingSocket::ReadText(char* pch, const int nSize, const int nSecs)
// reads remainder of a transmission through buffer full or socket close
// (assume headers have been read already)
{
	int nBytesToRead, nBytesThisTime, nBytesRead = 0;
	if(m_nReadBuf > 0) { // copy anything already in the recv buffer
		memcpy(pch, m_pReadBuf, m_nReadBuf);
		pch          += m_nReadBuf;
		nBytesRead    = m_nReadBuf;
		m_nReadBuf    = 0;
	}
	do { // now pass the rest of the data directly to the caller
		nBytesToRead = min(nSizeRecv, nSize - nBytesRead);
		if(nBytesToRead > 0) {
			nBytesThisTime = Receive(pch, nBytesToRead, nSecs);
			if(nBytesThisTime <= 0) {
				break; // peer closed the socket
			}

			pch        += nBytesThisTime;
			nBytesRead += nBytesThisTime;
		}
	}
	while(nBytesRead < nSize);
	memset(m_pReadBuf,0,5000);
	return nBytesRead;
}


void SocketError(wchar_t * pchMessage)
{
	
	int nError = WSAGetLastError();
	wchar_t text[255];
	if(nError == 0) {
		wsprintf(text, L"WINSOCK ERROR--%s error", pchMessage);
	}
	else {
		wsprintf(text, L"WINSOCK ERROR--%s error #%d", pchMessage, nError);
	}

	MessageBox(hwndMain,text,NULL,MB_OK);

	PWCHAR pwcNewText = L"Connect";
	MENUITEMINFO mii;
	memset((char *)&mii, 0, sizeof(mii));
	mii.cbSize = sizeof(mii); 
	mii.fMask  = MIIM_DATA | MIIM_TYPE; 
	mii.fType = MFT_STRING;
	mii.dwTypeData = pwcNewText;
	SetMenuItemInfo(hMenu,IDM_HELP_CONNECT, TRUE, &mii);

}


void SocketWarning(wchar_t * pchMessage)
{
	
	int nError = WSAGetLastError();

	wchar_t text[255];
	if(nError == 0) {
		wsprintf(text, L"WINSOCK ERROR--%s error", pchMessage);
	}
	else {
		wsprintf(text, L"WINSOCK ERROR--%s error #%d", pchMessage, nError);
	}
}


