
#ifndef __BLOCKSOCK_H__
#define __BLOCKSOCK_H__

// blocksock.h
//#include "sslsock.h"
#include "winsock.h"
#include <winuser.h>
#include <commctrl.h>
#include <windows.h>

#ifdef _WIN32_WCE
#include <atlbase.h>
//#include <wceatl.h> 
#else
#include <crtdbg.h>
#endif

// needs winsock.h in the precompiled headers

typedef const struct sockaddr* LPCSOCKADDR;

extern void SocketError(wchar_t *pError);
extern void SocketWarning(wchar_t *pError);

class CSockAddr : public sockaddr_in {
public:
	// constructors
	CSockAddr()
		{ sin_family = AF_INET;
		  sin_port = 0;
		  sin_addr.s_addr = 0; } // Default
	CSockAddr(const SOCKADDR& sa) { memcpy(this, &sa, sizeof(SOCKADDR)); }
	CSockAddr(const SOCKADDR_IN& sin) { memcpy(this, &sin, sizeof(SOCKADDR_IN)); }
	CSockAddr(const ULONG ulAddr, const USHORT ushPort = 0) // parms are host byte ordered
		{ sin_family = AF_INET;
		  sin_port = htons(ushPort);
	      sin_addr.s_addr = htonl(ulAddr); }
	CSockAddr(const char* pchIP, const USHORT ushPort = 0) // dotted IP addr string
		{ sin_family = AF_INET;
		  sin_port = htons(ushPort);
		  sin_addr.s_addr = inet_addr(pchIP); } // already network byte ordered
	// Return the address in dotted-decimal format
	//CString DottedDecimal()
	//	{ return inet_ntoa(sin_addr); } // constructs a new CString object
	// Get port and address (even though they're public)
	USHORT Port() const
		{ return ntohs(sin_port); }
	ULONG IPAddr() const
		{ return ntohl(sin_addr.s_addr); }
	// operators added for efficiency
	const CSockAddr& operator=(const SOCKADDR& sa)
		{ memcpy(this, &sa, sizeof(SOCKADDR));
		  return *this; }
	const CSockAddr& operator=(const SOCKADDR_IN& sin)
		{ memcpy(this, &sin, sizeof(SOCKADDR_IN));
		  return *this; }
	operator SOCKADDR()
		{ return *((LPSOCKADDR) this); }
	operator LPSOCKADDR()
		{ return (LPSOCKADDR) this; }
	operator LPSOCKADDR_IN()
		{ return (LPSOCKADDR_IN) this; }
};

// member functions truly block and must not be used in UI threads
// use this class as an alternative to the MFC CSocket class
class CBlockingSocket 
{

public:
	SOCKET m_hSocket;
	BOOL m_bSocketCreated;
	BOOL m_bSocketBusyWrite; // jek: Flag indicating that a write_thread writes to the socket
	BOOL m_bSocketBusyRead;  // jek: Flag indicating that a read_thread reads from the socket
	HWND m_hWnd;
	CBlockingSocket() { m_hSocket = NULL; m_bSocketCreated = false;}
	void Cleanup();
	void Create(int nType = SOCK_STREAM);
	void Close();
	void Bind(LPCSOCKADDR psa);
	void Listen();
	int  Connect(LPCSOCKADDR psa);
	BOOL Accept(CBlockingSocket& s, LPSOCKADDR psa);
	int Send(const char* pch, const int nSize, const int nSecs);
	int Write(const char* pch, const int nSize, const int nSecs);
	int Receive(char* pch, const int nSize, const int nSecs);
	int CBlockingSocket::ReadNbytes(void *buf, int n, int wait);
	int SendDatagram(const char* pch, const int nSize, LPCSOCKADDR psa, 
		const int nSecs);
	int ReceiveDatagram(char* pch, const int nSize, LPSOCKADDR psa, 
		const int nSecs);
	void GetPeerAddr(LPSOCKADDR psa);
	void GetSockAddr(LPSOCKADDR psa);
	static CSockAddr GetHostByName(const char* pchName, 
		const USHORT ushPort = 0);
	static const char* GetHostByAddr(LPCSOCKADDR psa);
	static CSockAddr MUST_GetHostByAddr(char *ipaddr, const USHORT ushPort);
    static u_long ToIPAdr(char *hname); 

	operator SOCKET()
		{ return m_hSocket; }
};









class CHtmlBlockingSocket : public CBlockingSocket
{
public:
	enum {nSizeRecv = 5000}; // max receive buffer size (> hdr line length)

	// jek:
	// socketStatusn == 0 : Disconnected
	// socketStatusn == 1 : Connected
	// socketStatusn == 2 : Error condition during last attemt to make a connection
	int socketStatus; // use g_sClient.m_bSocketCreated instead if only connected/disconnected is checked
	
	CHtmlBlockingSocket();
	~CHtmlBlockingSocket();
	int ReadLine(char* pch, const int nSize, const int nSecs);
	int ReadText(char* pch, const int nSize, const int nSecs);
private:
	char* m_pReadBuf; // read buffer
	int   m_nReadBuf; // number of bytes in the read buffer
};

#endif
