#include "vad.h"

int dbg_=0;


/*
Constructor
*/
Vad::Vad()
{
	
	speechthresh = 9.0; //thres. for speech above sil level
	silenergy    = 50.0; // Average bg noise level dB (for ipaq-builtin microphone  50.0)
	//spcseqcount  = 10 ; // window over which speech/silence decision
	spcseqcount  = 16 ; 
	spcglchcount = 0;// max sil frames before speech onset
	silseqcount  = 5; // org ipaq= 3 number of sil windows for end of speech
	//silglchcount = 2; // number of sil frames before end of speech
	silglchcount = 6;  //org ipaq=4
	//silmargin    = 10; //margin in frames
	silmargin    = 20; //org ipaq =16
	
	sampling_rate = 8000; // In hertz
	buffer_size  = 4096; // In bytes
	//frame_length = 160; // in samples per frame (~ 20msec at 8kHz)
	frame_length =128; // 16msec at 8kHz 
	
	frame_buffer_size= 254*4;
	
	///// 
	samp_per_window= spcseqcount*frame_length;
	
	sample_buffer = new
		char[2*((silseqcount+1)*frame_length*spcseqcount)];
	sample_buffer_pos=sample_buffer;
	//decision_window= new int[spcseqcount];
	
	speech_frame_cnt=0;
	silence_window_cnt=0;
	status=0;
	window_status_current=0;
	
	window_cnt=0;
	
	
}

/*
Destructor

*/
Vad::~Vad()
{
	delete [] sample_buffer;
	//delete [] decision_window;
}



int Vad::IsSpeech(char *windo, int len)
{
	
	if (len != 2*samp_per_window)
    {
	    MessageBox(NULL,L"Len value is wrong",L"Error", MB_OK);
		//      cerr << "len should be " << 2*samp_per_window << endl;
		exit(0);
    }
	
	//cerr << "\nWindow count = " << window_cnt++ << endl;
	short *windo_short= (short *)windo;
	speech_frame_cnt=0;
    
	for (int frame_nr=0;frame_nr < spcseqcount; frame_nr++)
    {
		int start_s= frame_nr*frame_length;
		int end_s =  (frame_nr+1)*frame_length-1;
		double energy=0;
		
		
		//calculate the mean energy
		for (int i=start_s;i<=end_s;i++){
			//cerr << windo_short[i]  << " " ;
			energy += windo_short[i]* windo_short[i];
		}
		//cerr << "\n";
		
		energy=10*LOG10((double)energy/(double)frame_length);
		//cerr << "\t energy= " << energy << "\n";
	//	printf("Energy = %f\n", energy);
		//determine whether frame is silence or speech
		if (energy > silenergy+ speechthresh)
			speech_frame_cnt++;
    }
	// cerr << "\tspeech_frame_cnt= " << speech_frame_cnt << "\n";
	//determine if the window is silence or speech
	if (speech_frame_cnt >= spcseqcount - silglchcount)
		window_status_current=1;
	else
		window_status_current=0;
	
	//cerr << "Buffer pos = " << (sample_buffer_pos-sample_buffer) << endl;
	//Add the current window to buffer
	for (int i=0;i<len; i++){
		*(sample_buffer_pos+i)=windo[i];
	}
	sample_buffer_pos += len;
	
	
	//Determine the return value according to the heuruistics in HTKBook
	switch(status)
    {
    case 0:
		MSG_("CASE 0 ");
		//speech on-set detected
		if ( window_status_current==1){
			MSG_("\tSPEECH");
			status=1;
			buffered_samples= 2*len;
			sample_buffer_pos=sample_buffer;
			return 1;
		}
		//speech on-set yet to be detected
		else {
			MSG_("\tNON SPEECH");
			sample_buffer_pos =sample_buffer+len;
			return -2;
		}
		
		
    case 1:
		MSG_("CASE 1");
		//speech 
		if (window_status_current ==1){
			MSG_("\tSPEECH");
			sample_buffer_pos=sample_buffer;
			return 0;
		}
		//silence window
		else {
			MSG_("\tNON SPEECH");
			status=2;
			silence_window_cnt++;
			
			return -1;
		}
		
    case 2:
		MSG_("CASE 2");
		if (window_status_current == 1){
			MSG_("\tSPEECH");
			silence_window_cnt=0;
			buffered_samples=sample_buffer_pos-sample_buffer;
			sample_buffer_pos=sample_buffer;
			status=1;
			return 2;
		}
		else if (silence_window_cnt < silseqcount){
			MSG_("\tNON SPEECH, but within SIL COUNT");
			status=2;
			silence_window_cnt++;
			return -1;
		}
		else if (silence_window_cnt == silseqcount){
			MSG_("\tNON SPEECH but SIL COUNT Exceeded");
			status=0;
			silence_window_cnt=0;
			window_cnt=0;
			buffered_samples=len+ (2*silmargin*frame_length);
			sample_buffer_pos=sample_buffer;
			return 3;
		}
		else {
				MessageBox(NULL,L"Status that cannot be handled",L"Error", MB_OK);
			exit(0);
		}
		
    default:
		      MessageBox(NULL,L"wrong status",L"Eror", MB_OK);
		exit(0);
		
		
		
    }
	
}



/*
Return the pointer to sample buffer
*/
char *Vad::getSpeechWindows(){
	return sample_buffer;
}

/*
Return the size of buffered samples in bytes
*/
int Vad::getSpeechWindowsLen(){
	return buffered_samples;
}







