/* 
You must link with winmm.lib. If using Visual C++, go to Build->Settings. Flip to the Link page,
and add winmm.lib to the library/object modules.

This app shows how to use the Low Level Wave API to play the WAVE file C:\Windows\Chord.wav.
It also uses the MMIO (RIFF) File Parsing functions.
*/

#include "wavply.h"
#ifndef __WAVPLY_CPP_
#define __WAVPLY_CPP_

//---------------------------------------
//Constructor
//---------------------------------------
WavPly::WavPly(/*char*/ socket_stream *server_soc /*ip_address*/, HWND parent_wnd)
{
	WaveFormat.wFormatTag = WAVE_FORMAT_PCM;
	WaveFormat.nChannels = 1;
	WaveFormat.nSamplesPerSec = 8000;
	WaveFormat.wBitsPerSample = 16;
	WaveFormat.nBlockAlign = WaveFormat.nChannels * (WaveFormat.wBitsPerSample >> 3);
	WaveFormat.nAvgBytesPerSec = WaveFormat.nSamplesPerSec * WaveFormat.nBlockAlign;
	WaveFormat.cbSize = 0;


	WavDataRead=0;
	pl_state=0;
	delimiter_pos=0;
	WaveBufSize=16*128;//1600; //Given in #ALAW_BYTES
	nQueued=0;
	QueueSize=10;
	playStopped=0;
	playStopped_forced=0;

    WaveBufSize_pcm= WaveBufSize << 1; //Twice the ALAW size
    wavbuf_alaw = new char[WaveBufSize];
	residue= new char[WaveBufSize]; //Reads alaw from the socket
	buf_count=0;

	parent_window =parent_wnd;

#ifdef _RDF_SOCKET
	sock_str = server_soc; // new socket_stream(server_ip_address);
#endif

	if ( ! (Event = CreateEvent(0, TRUE, FALSE, 0)))
	{
		MessageBox(NULL, L"cannot create event",L"Error",MB_OK);
		exit(0);
	}

/*  if ((hLogFile = 
	 CreateFile(L"FT1.alw",GENERIC_WRITE,0,NULL,
						CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL)) 
						== INVALID_HANDLE_VALUE){
 	MessageBox(NULL, TEXT("Cant open FT1.alw"), TEXT("Error"), MB_OK); 
	exit(0);
  }
*/

	LogFile= fopen("play_log.txt", "w");
	VoipLog_fd = NULL; // To be set by voip

#ifdef _RDF_FILE
 if ((hAlawFile = CreateFile("FT.alw",	GENERIC_READ,
		0,NULL,OPEN_EXISTING,	FILE_ATTRIBUTE_NORMAL,NULL))
		== INVALID_HANDLE_VALUE){
	 	MessageBox(NULL, TEXT("Cant open FT.alw"), TEXT("Error"), MB_OK); 
	exit(0);
	}
#endif
}


//----------------------
//Destructor
//---------------------
WavPly::~WavPly()
{   
	CloseHandle(Event);
//	CloseHandle(hLogFile);
	fclose(LogFile);
	delete [] residue;
	delete [] wavbuf_alaw;
#ifdef _RDF_SOCKET
	//delete sock_str;
#endif
}

//----------------------------------------------
//In case the sampling rate is wanted to be set
//----------------------------------------------
int WavPly::SetSampRate(int r)
{
	WaveFormat.nSamplesPerSec = r;
	return 0;
}



//----------------------------------------------
// FSM implementation of reading from the socket 
// The reading FSM has three states
// Routine includes a reading from a file as well
// but it is not an FSM
//----------------------------------------------
int WavPly::AquireData(WAVEHDR *waveHeader)
{
	DWORD err;	
	int soc_read_res;
	short int  * wavdata_buf =(short int *)waveHeader->lpData;


	time_out=0;
	buf_type=NORMAL;

	int tul;

#ifdef _RDF_SOCKET
    //We have not started reading the a signal yet 
	switch (pl_state){
		
	case 0: 
		//WavDataRead=0; //reset this counter
		if (WavDataRead != 0)
			//strcpy((char *)waveHeader->lpData, residue);
			strcpy(wavbuf_alaw, residue);
		soc_read_res=sock_str->read(WavDataRead+&wavbuf_alaw[0]/*(char *)waveHeader->lpData*/, WaveBufSize-WavDataRead, 0);
		buf_count=0;
#ifdef NO_BARGEIN
			    //Get MUTEX
		        //sync_obj->setPlaying(1);
		if (sync_obj->isPlaying()){
			if (MMSYSERR_NOERROR == waveOutReset(HWaveOut)){
				tul=0;
				while (sync_obj->isPlaying()){
					if (tul++ > 50)
						break;
					if (verbosed ==1)
						fprintf(VoipLog_fd, "waiting for playing to be finished\n");
					Sleep(5);
				}
			}
			else{
				MessageBox(NULL, L"Cannot reset the waveout device", L"Error", MB_OK);
				exit(0);
			}
		}
		buf_type = START;
				
#endif
        if(soc_read_res <=0){ 
			/* Stop playback */
			//waveOutPause(HWaveOut);
#ifdef _MSG_TO_SCREEN_
			if (soc_read_res < 0){
				wchar_t  tmp_msg[20];
				swprintf(tmp_msg, L"socket failure_1 %d", soc_read_res);
				MessageBox(NULL, tmp_msg /*TEXT("ERROR: Socket Failure_1")*/, TEXT("Error"),MB_OK);
			}
#endif
			return(soc_read_res);
		}
		else if (soc_read_res >0 /*<= WaveBufSize*/){ //read but not fully
			pl_state=1;
			//Reset the rest of the buffer
			for (int j=soc_read_res+WavDataRead;j<WaveBufSize; j++)
				/*((char *)*/wavbuf_alaw[j]/*waveHeader->lpData)[j]*/='\x55';
			WavDataRead=WavDataRead+soc_read_res;
			delimiter_pos=WaveBufSize-WavDataRead%WaveBufSize;
			if (delimiter_pos == WaveBufSize) delimiter_pos=0;
			
		}
		break;
		
		
		//Reading happens within a speech signal
	case 1: 
		//time_out=0;
		soc_read_res=sock_str->read(&wavbuf_alaw[0]/*(char *)waveHeader->lpData*/, WaveBufSize, 200, (void *)&time_out);
        if(soc_read_res <=0){ 
			/* Stop playback */	
			//waveOutPause(HWaveOut);
			if (soc_read_res <0)
				MessageBox(NULL, TEXT("ERROR: Socket Failure Within Signal"), TEXT("Error"),MB_OK);
//			printf("socket failure %d\n", soc_read_res);
//			exit(0);
			return(soc_read_res);
		}
		else if (soc_read_res <= delimiter_pos){ // not enough data to determine the EOS (end of signal)
			pl_state=1;
			//WavDataRead=WavDataRead+soc_read_res;
			//delimiter_pos=WaveBufSize-WavDataRead%WaveBufSize;
			//if (delimiter_pos == WaveBufSize) delimiter_pos=0;
            
		}
        else if (soc_read_res == delimiter_pos+1) { //gives only the first byte of the EOS-signature
			if ((unsigned char)/*wavdata_buf*/wavbuf_alaw[delimiter_pos] == '\x2a'){ //first byte match
				/*wavdata_buf*/wavbuf_alaw[delimiter_pos] == '\x55';
				pl_state=2; //We need another state
			}
			else
				pl_state=1;  //We carry on as usual
			//WavDataRead=WavDataRead+soc_read_res;
			
		}
		else if (soc_read_res > delimiter_pos+1){ //Read enough data		
			
			//Check the signature match
			if ( /*wavdata_buf*/wavbuf_alaw[delimiter_pos] == '\x2a' &&
				/*wavdata_buf*/wavbuf_alaw[delimiter_pos+1] == '\xaa'){ //Start a new signal if matched
				/*wavdata_buf*/wavbuf_alaw[delimiter_pos] = '\x55';
				/*wavdata_buf*/wavbuf_alaw[delimiter_pos+1] = '\x55';
				pl_state=0;  
#ifdef NO_BARGEIN
			    //Release MUTEX
			    //sync_obj->setPlaying(0);
			     buf_type=LAST;
#endif
				if (soc_read_res >= delimiter_pos+2){
					WavDataRead=soc_read_res-delimiter_pos-2;
					int off_set=delimiter_pos+2;
					for (int j=0;j<WavDataRead;j++){
						residue[j]=/*wavdata_buf*/wavbuf_alaw[j+off_set];
						/*wavdata_buf*/wavbuf_alaw[j+off_set]='\x55';		
					}
					residue[WavDataRead]='\0';
				}
			}
			else{  //Carry on as usual if not
				pl_state=1; 
			}
			
			//WavDataRead=WavDataRead+soc_read_res;
			//delimiter_pos=WaveBufSize-WavDataRead%WaveBufSize;
			//Reset the rest of the buffer
			//for (int j=soc_read_res;j<WaveBufSize; j++)
			//	(char *)waveHeader->lpData[j]='\x00';
			
		}	
		if (pl_state !=0){
			WavDataRead=WavDataRead+soc_read_res;
			delimiter_pos=WaveBufSize-WavDataRead%WaveBufSize;
			if (delimiter_pos == WaveBufSize) delimiter_pos=0;
		}
		//Reset the rest of the buffer
		{	for (int j=soc_read_res;j<WaveBufSize; j++)
			/*((char *)waveHeader->lpData)*/wavbuf_alaw[j]='\x55';}
		break;
		
		// This is an unlikely state where only a single EOS-signature byte has been read
	case 2:
		soc_read_res=sock_str->read(&wavbuf_alaw[0]/*(char *)waveHeader->lpData*/, WaveBufSize, 20);
        if(soc_read_res <=0){ 
			/* Stop playback */
			//waveOutPause(HWaveOut);
			if (soc_read_res < 0)
				MessageBox(NULL, TEXT("ERROR: Socket Failure_2"), TEXT("Error"),MB_OK);
			return(soc_read_res);
		}
		else if (soc_read_res >0){ 
			
			if ( (unsigned char)/*wavdata_buf*/wavbuf_alaw[delimiter_pos] == '\xaa'){ //EOS-signature match
				pl_state=0;
#ifdef NO_BARGEIN
			    //Release MUTEX
				//sync_obj->setPlaying(0);
				buf_type=LAST;
#endif
				if (soc_read_res > delimiter_pos+1){
					WavDataRead=soc_read_res-delimiter_pos-1;
					int off_set=delimiter_pos+1;
					for (int j=0;j<WavDataRead;j++){
						residue[j]=/*wavdata_buf*/wavbuf_alaw[j+off_set];
						/*wavdata_buf*/wavbuf_alaw[j+off_set]='\x55';		
					}
				}
				
			}
			else { // Carry out as usual
				pl_state=1;
				WavDataRead=WavDataRead+soc_read_res;
				delimiter_pos=WaveBufSize-WavDataRead%WaveBufSize;
				if (delimiter_pos == WaveBufSize) delimiter_pos=0;
				//Reset the rest of the buffer
				for (int j=soc_read_res;j<WaveBufSize; j++)
					/*((char *)waveHeader->lpData)*/wavbuf_alaw[j]='\x55';
			}
		}
		break;
		
	}
	
	
	//   DWORD written;
	 //  if (!WriteFile(hLogFile,/*wavdata_buf*/wavbuf_alaw,WaveBufSize,&written,NULL)){
	//	   MessageBox(NULL,TEXT("cant write to the log file"),TEXT("Error"),MB_OK);
	//	   exit(0);
	  // }
	
	   
#endif
	   
	   
	   
#ifdef _RDF_FILE
	   int read_res= ReadBytesFromFile(&hAlawFile, /*wavdata_buf*/wavbuf_alaw, WaveBufSize);
	   if (read_res==0){
		   DWORD seek_res= SetFilePointer(hAlawFile,0,NULL, FILE_BEGIN);
		   if (seek_res== 0xFFFFFFFF){
			   MessageBox(NULL,TEXT("cant move the file pointer"),TEXT("Error"),MB_OK);
			   exit(0);
		   }
		   
	   }
#endif
       //Convert alaw to 16 bit PCM
	   for (int k=0;k<WaveBufSize;k++)
		   wavdata_buf[k]= AlawDecTab[(unsigned char)wavbuf_alaw[k]];

	   buf_count++;  //Increase the buffers read.
	   
	   if (verbosed){
		   if (buf_type == LAST){
			   fprintf(LogFile, "Buffers read = %d\n", buf_count);
			   fprintf(VoipLog_fd, "WavPlay: buffers read = %d\n", buf_count);
		   }
	   }

//   DWORD written;
//   if (!WriteFile(hLogFile,/*wavdata_buf*/wavdata_buf,WaveBufSize_pcm,&written,NULL)){
//		   MessageBox(NULL,TEXT("cant write to the log file"),TEXT("Error"),MB_OK);
//		   exit(0);
//	   }
 //    
	   return 1;
}


//---------------------------------------------------------
// Queues a buffer worth of data for the passed WAVEHDR. 
// REturns 10, if the playback should continue.
// Note that this can queue up to QueueSize buffers
//---------------------------------------------------------
int WavPly::QueueWaveData(WAVEHDR * waveHeader)
{
    /*DWORD*/ int err=0;
	
	
	
	err=AquireData(waveHeader);
#ifdef _MSG_TO_SCREEN_
	wchar_t tmp_m[36];
	swprintf(tmp_m, L" going to try to post soc fail msg %d", err);
	MessageBox(NULL, tmp_m, L"Error", MB_OK);
#endif
    // Socket failures should not be tolerated
	if (err <0) { //Socket error
#ifdef _MSG_TO_SCREEN_
		MessageBox(NULL, L"trying post soc fail msg", L"Error", MB_OK);
#endif
        if (parent_window != NULL) {//Notifies this the parent window
			PostMessage(parent_window, WM_USER+12, NULL,NULL);
#ifdef _MSG_TO_SCREEN_
			MessageBox(NULL, L"posted soc fail msg", L"Error", MB_OK);
#endif
		}
	}

	if (err <=0){  //Socket failure
		if (verbosed == 1){
			fprintf(VoipLog_fd, "WavPlay: last buffer..\n");
		}
        if (time_out==1)
			return 10;
		short int *wav_buf=(short int *)waveHeader->lpData;
		for (int k=0;k<WaveBufSize;k++)
			wav_buf[k]=(short int)-8;
		if (verbosed == 1){
			fprintf(VoipLog_fd, "WavPlay: last buffer -msg 2\n");
		}
		waveHeader->dwUser=1;
		playStopped = 1; //Sign for play_stop
		if (parent_window != NULL) //Notifies this the parent window
			PostMessage(parent_window, WM_USER+10, NULL,NULL);

	}
#ifdef NO_BARGEIN
	else {
		if (buf_type == START){
			if (verbosed == 1){
				 fprintf(VoipLog_fd, "WavPlay: notifying start buffer\n");
			}
			 sync_obj->setPlaying(1);
		}
		 waveHeader->dwUser=buf_type;
	}		
#endif
	
	if (nQueued > QueueSize){
		WaitForSingleObject(Event, INFINITE);
		ResetEvent(Event);
	}

    
	
	if (err=waveOutPrepareHeader(HWaveOut, waveHeader, sizeof(WAVEHDR)))
	{		
		DisplayLastError();
		return -1;
	}
	
	// Queue the WAVEHDR 
	if (err=waveOutWrite(HWaveOut, waveHeader, sizeof(WAVEHDR))){
		DisplayLastError(); 
		return (-1);
	}

//	printf("Queued\n");
	//Sleep(5000);
	
    nQueued++; 
	
	return(10);	
	
	
}

//-----------------------------------------------------------------
//Plays the WAVE, by calling queuewave repeatedly
//--------------------------------------------------------------
int WavPly::PlayWave(/*char *filenm*/)
{

	DWORD	err,err_q;
	int ret_val;


	/* Open the default WAVE Out Device, specifying my callback */
	if ((err = waveOutOpen(&HWaveOut, WAVE_MAPPER, &WaveFormat, (DWORD)(::WaveOutProc), (DWORD)this, CALLBACK_FUNCTION)))
	{
		MessageBox(NULL, TEXT("ERROR: Can't open WAVE Out Device"), TEXT("Error"), MB_OK);
		return -1;
	}



    m_hCleanThread=CreateThread(NULL,NULL,cleanThread,(void*)this,NULL,&id_CleanThread);

	while (1){
		//if (playStopped)
		//	break;
		
		newWaveHeader = (WAVEHDR *)new BYTE[sizeof(WAVEHDR)+WaveBufSize_pcm];
		newWaveHeader->lpData= (char *)&newWaveHeader[1]; 
		
		
		/* Fill in WAVEHDR fields for buffer starting address and size */
		newWaveHeader->dwFlags =0;
		newWaveHeader->dwLoops =0;
		newWaveHeader->dwBufferLength = WaveBufSize_pcm;
		if (playStopped_forced==1){
			newWaveHeader->dwUser=1;
			playStopped=1;
		}
		else
			newWaveHeader->dwUser=0;
		
		err_q=QueueWaveData(newWaveHeader);
	//	delete newWaveHeader;
		if (err_q != 10 || playStopped==1)
			break;
	}
   

//	printf("waiting for the thread to finish\n");
    if (err_q == 10){ //playStopped
		WaitForSingleObject(m_hCleanThread, INFINITE);
		ret_val = 0;
	}
	else //Error
		ret_val= -1;

	//Stop Windows queuing of buffers (and calling of my callback) 
	waveOutReset(HWaveOut);

    //Close WAVE Out device 
	waveOutClose(HWaveOut);
//    printf("returning\n");
    return ret_val; 
}


//---------------------------------------------------
// This routine run in a sperate thread, takes care of 
// the garbage handling of the returned (playbacked) waveheaders
// from the waveOut device
//--------------------------------------------------- 
int WavPly::CleanWave()
{
	MSG msg;
	short done=0;

	while (GetMessage(&msg, NULL, 0, 0)) 
	{
//		printf("received message\n");
		if (msg.message==WM_USER+501)
		{
            LPWAVEHDR whd= (LPWAVEHDR)msg.lParam; 
			if (/*((LPWAVEHDR)msg.lParam)*/whd->dwUser == 1)
				done=1;
			else if (whd->dwUser == LAST){
				if (verbosed == 1){
					fprintf(VoipLog_fd, "WavPlay: last buffer played\n");
				}
				sync_obj->setPlaying(0);
			}
			waveOutUnprepareHeader( HWaveOut, (LPWAVEHDR)msg.lParam, sizeof(WAVEHDR) );
			nQueued--;
			
			if (done==0 && nQueued==QueueSize) SetEvent(Event);        
			delete (void*)msg.lParam;
		}
		else if (msg.message==WM_QUIT)
		{
			return 0;
		}
		TranslateMessage(&msg);
		DispatchMessage(&msg);

		if (done ==1)
			break;
	}
	return 0;
}


//-------------------------------------------
//Interface to the external world, which starts the 
// playing thread
//----------------------------------------------
void WavPly::startPlay()
{
	//We make a fresh start
	pl_state=0;
	playStopped=0;
	playStopped_forced=0;

    m_hPlayThread = CreateThread(NULL,
								0,
								&playThread,
								(LPVOID) this,
								// CREATE_SUSPENDED,
								0,
								NULL);
#ifndef _WIN32_WCE
	//WaitForSingleObject(m_hPlayThread, INFINITE);
#endif
	//printf("thread exits");

}


//----------------------------------------------
//Stops the playing thread
//----------------------------------------------
void WavPly::stopPlay(){
	playStopped_forced=1;

}

//----------------------------------------
//Waits until the play thread is completed
//---------------------------------------
void WavPly::waitPlay(){
	if (verbosed == 1){
		fprintf(VoipLog_fd, "WavPlay: waitplay enter\n");
	}
	WaitForSingleObject(m_hPlayThread, INFINITE);
	if (verbosed == 1){
		fprintf(VoipLog_fd, "WavPlay: waitplay exit\n");
	}
}

//------------------------------------------
// Adds a sync object
//---------------------------------------------
void WavPly::addSync(Sync *s){
	sync_obj=s;
}


//------------------------------------------
// Sets the verbosity
//--------------------------------------
void WavPly::setVerbosity(int v){
	verbosed=v;
}

//---------------------------------------
// Sets the common logfile for play and record
//--------------------------------------
void WavPly::setLogFD(FILE *fd){
	VoipLog_fd= fd;
}

//-------------------------------------------
// Callback function for the WaveOut device
//-------------------------------------------
void  CALLBACK WaveOutProc(HWAVEOUT waveOut, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2)
{
//	printf("glb called\n");
//	cerr << "Global WaveOutProc called\n";
	if (uMsg == WOM_DONE){
		WavPly *wplayobj =(WavPly *) dwInstance;
		LPWAVEHDR pw= (LPWAVEHDR) dwParam1;
		PostThreadMessage(wplayobj->id_CleanThread,WM_USER+501,NULL,dwParam1);
	}

}

//-------------------------------------
// Auxliary thread caller
//-----------------------------------
DWORD WINAPI playThread(VOID *vptr)
{
	WavPly *wplayobj = (WavPly*)vptr;
    if (wplayobj->PlayWave() <0){
		MessageBox(NULL, L"Could not call playwave", L"Error", MB_OK);
		exit(0); 
	}
	return 1;
}

//--------------------------------------
// Auxiliary thread caller
//-----------------------------------
DWORD WINAPI cleanThread(VOID *vptr)
{
	((WavPly*)vptr)->CleanWave();
	return 1;
}


#endif


/* ******************************** main() ******************************** */

#ifdef __STANDALONE_PLAY

int main(int argc, char **argv)
{
//	if (argc != 3){
//		cerr << "Wavply plays 8/16Khz, 16 bit PCM files\n";
//		cerr << "USAGE : wavply pcmfilename  8000/16000\n";          
//		exit(0);
//	}

WavPly	*wpobj = new WavPly(new socket_stream("127.0.0.1"),NULL);//"148.121.142.16");
	//wpobj->SetSampRate(atoi(argv[2]));
	//wpobj->PlayWave(argv[1]);
	wpobj->startPlay();
    
	//Sleep(100000);
	//wpobj->stopPlay();

	return(-1);
}

#endif