/* 
You must link with winmm.lib. If using Visual C++, go to Build->Settings. Flip to the Link page,
and add winmm.lib to the library/object modules.

This app shows how to use the Low Level Wave API to play the WAVE file C:\Windows\Chord.wav.
It also uses the MMIO (RIFF) File Parsing functions.
*/

#include "wavrec.h"
#include "wavply.h"
#ifndef __WAVREC_CPP_
#define __WAVREC_CPP_



//---------------------------------------
//Constructor
//---------------------------------------
WavRec::WavRec(/*char*/ socket_stream *server_soc /*ip_address*/, HWND parent_wnd)
{
	WaveFormat.wFormatTag = WAVE_FORMAT_PCM;
	WaveFormat.nChannels = 1;
	WaveFormat.nSamplesPerSec = 8000;
	WaveFormat.wBitsPerSample = 16;
	WaveFormat.nBlockAlign = WaveFormat.nChannels * (WaveFormat.wBitsPerSample >> 3);
	WaveFormat.nAvgBytesPerSec = WaveFormat.nSamplesPerSec * WaveFormat.nBlockAlign;
	WaveFormat.cbSize = 0;


	WavDataWritten=0;
	pl_state=0;
	delimiter_pos=0;
	WaveBufSize=16*128;//1600; //Given in #ALAW_BYTES
	nQueued=0;
	QueueSize=10;
	recStopped=0;

	mic_off=0;

    WaveBufSize_pcm= WaveBufSize << 1; //Twice the ALAW size
    wavbuf_alaw = new char[WaveBufSize];
	residue= new char[WaveBufSize]; //Reads alaw from the socket
	spdet = new Vad();

	parent_window =parent_wnd;

#ifdef _WRT_SOCKET
	sock_str = server_soc; // new socket_stream(server_ip_address);
#endif

	if ( ! (Event = CreateEvent(0, TRUE, FALSE, 0)))
	{
		MessageBox(NULL, L"cannot create event",L"Error",MB_OK);
		exit(0);
	}
 
	
/*	if ((hLogFile = 
	 CreateFile(L"FT2.alw",GENERIC_WRITE,0,NULL,
						CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL)) 
						== INVALID_HANDLE_VALUE){
 	MessageBox(NULL, TEXT("Cant open FT2.alw"), TEXT("Error"), MB_OK); 
	exit(0);
  }
*/
      VoipLog_fd = NULL; // To be set by voip

#ifdef _WRT_FILE
 if ((hAlawFile = CreateFile("FT.alw",	GENERIC_WRITE,
		0,NULL,CREATE_ALWAYS,	FILE_ATTRIBUTE_NORMAL,NULL))
		== INVALID_HANDLE_VALUE){
	 	MessageBox(NULL, TEXT("Cant open FT.alw"), TEXT("Error"), MB_OK); 
	exit(0);
	}
#endif
}


//----------------------
//Destructor
//---------------------
WavRec::~WavRec()
{   
	CloseHandle(Event);
//	CloseHandle(hLogFile);
	delete [] residue;
	delete [] wavbuf_alaw;
	delete spdet;
#ifdef _WRT_SOCKET
	//delete sock_str;
#endif
}

//----------------------------------------------
//In case the sampling rate is wanted to be set
//----------------------------------------------
int WavRec::SetSampRate(int r)
{
	WaveFormat.nSamplesPerSec = r;
	return 0;
}



//----------------------------------------------
// FSM implementation of reading (writing to ??) from the socket 
// The reading FSM has three states
// Routine includes a reading from a file as well
// but it is not an FSM
//----------------------------------------------
int WavRec::ReleaseData(WAVEHDR *waveHeader)
{
	//DWORD err;	
	int soc_write_res;
	int vad_res;
	short int  * wavdata_buf =(short int *)waveHeader->lpData;
	short int  * wavdata_buf_;
	char       * wavbuf_alaw_;
	int          WaveBufSize_;


	if (mic_off == 1)
		return 1;

#ifdef NO_BARGEIN
	if (sync_obj->isPlaying()){
		if (verbosed == 1){
	        fprintf(VoipLog_fd, "WavRec:: skip recording\n"); 
		}
		return 1;
	}
	else {
		if (verbosed == 1){
			fprintf(VoipLog_fd, "WavRec:: recording the buffer\n");
		}
	}
#endif
	
	vad_res=spdet->IsSpeech((char *)wavdata_buf, WaveBufSize_pcm);

	if (vad_res <0){
		return 1;   //Nothing to do
	}
	else if (vad_res == 0){ //go ahead and send this speech buffer
		wavdata_buf_=wavdata_buf;
		WaveBufSize_=WaveBufSize;
		wavbuf_alaw_=wavbuf_alaw;
	}
    else if (vad_res >0){ //Get buffered data from VAD
		wavdata_buf_=(short *)spdet->getSpeechWindows();
		int tmp_len =spdet->getSpeechWindowsLen();
		int len_in_samp=tmp_len >> 1;
        
		if (vad_res==3){ //Add EOS-signature
			*((len_in_samp -2)+wavdata_buf_) = -32256;
			*((len_in_samp -1)+wavdata_buf_) =32256;
		}
		WaveBufSize_ =len_in_samp;
		wavbuf_alaw_ =(char *)wavdata_buf_;

	}

//Convert from 16 bit PCM to ALAW
	   for (int k=0;k<WaveBufSize_;k++)
		   wavbuf_alaw_[k]= AlawEncTab[2048+ ((wavdata_buf_[k])>>4)];

	
#ifdef _WRT_SOCKET
    //We have not started reading the a signal yet 
//	   printf("wrt to socket\n");
	   soc_write_res= sock_str->write(wavbuf_alaw_, WaveBufSize_);
	   if (soc_write_res <=0) { // Error situation
		   return soc_write_res;
	   }
	   else if (soc_write_res < WaveBufSize_){ //Unlikely case in Blicking
            int remaining_data=WaveBufSize_-soc_write_res;
			char * rem_data_pt = wavbuf_alaw_+soc_write_res;

			while(1){
				soc_write_res=sock_str->write(rem_data_pt,remaining_data);
				if (soc_write_res <=0)
					return soc_write_res;
				else if (soc_write_res == remaining_data){
					break;
				}
				else{
					rem_data_pt=rem_data_pt+soc_write_res;
					remaining_data=remaining_data-soc_write_res;
				}
			}
         
	   }
//	   printf("written to sok\n");
	   
	   //Write to a log file as well
	   
//	   DWORD written;
//	   if (!WriteFile(hLogFile,/*wavdata_buf*/wavbuf_alaw_,WaveBufSize_,&written,NULL)){
//		   MessageBox(NULL,TEXT("cant write to the log file"),TEXT("Error"),MB_OK);
//		   exit(0);
//	   }
	   
#endif
	   
	   
	   
#ifdef _WRT_FILE
	   int read_res= ReadBytesFromFile(&hAlawFile, /*wavdata_buf*/wavbuf_alaw_, WaveBufSize_);
	   if (read_res==0){
		   DWORD seek_res= SetFilePointer(hAlawFile,0,NULL, FILE_BEGIN);
		   if (seek_res== 0xFFFFFFFF){
			   MessageBox(NULL,TEXT("cant move the file pointer"),TEXT("Error"),MB_OK);
			   exit(0);
		   }
		   
	   }
#endif
       
     
	   return 1;
}


//---------------------------------------------------------
// Queues a buffer worth of data for the passed WAVEHDR. 
// REturns 10, if the playback should continue.
// Note that this can queue up to QueueSize buffers
//---------------------------------------------------------
int WavRec::QueueWaveBuf(WAVEHDR * waveHeader)
{
    DWORD err;
		
//	printf("q buf\n");
	if (nQueued > QueueSize){
		WaitForSingleObject(Event, INFINITE);
		ResetEvent(Event);
	}
	
	if (err=waveInPrepareHeader(HWaveIn, waveHeader, sizeof(WAVEHDR)))
	{		
		DisplayLastError();
		return -1;
	}
	
	// Queue the WAVEHDR 
	if (err=waveInAddBuffer(HWaveIn, waveHeader, sizeof(WAVEHDR))){
		DisplayLastError(); 
		return (-1);
	}
	
	//waveInStart(HWaveIn);
    nQueued++; 

	
	
	return(10);	
	
	
}

//-----------------------------------------------------------------
//Plays (Records??) the WAVE, by calling queuewave repeatedly
//--------------------------------------------------------------
int WavRec::RecWave(/*char *filenm*/)
{

	DWORD	err,err_q;
	int ret_val;


	/* Open the default WAVE Out Device, specifying my callback */
	if ((err = waveInOpen(&HWaveIn, WAVE_MAPPER, &WaveFormat, (DWORD)(::WaveInProc), (DWORD)this, CALLBACK_FUNCTION)))
	{
		MessageBox(NULL, TEXT("ERROR: Can't open WAVE In Device"), TEXT("Error"), MB_OK);
		return -1;
	}



    m_hCleanThread=CreateThread(NULL,NULL,RecCleanThread,(void*)this,NULL,&id_CleanThread);
    waveInStart(HWaveIn);
	while (1){
		
		
		newWaveHeader = (WAVEHDR *)new BYTE[sizeof(WAVEHDR)+WaveBufSize_pcm];
		newWaveHeader->lpData= (char *)&newWaveHeader[1]; 
		
		
		/* Fill in WAVEHDR fields for buffer starting address and size */
		newWaveHeader->dwFlags =0;
		newWaveHeader->dwLoops =0;
		newWaveHeader->dwBufferLength = WaveBufSize_pcm;
	
		if (recStopped_forced==1){
			newWaveHeader->dwUser=1;
			recStopped=1;
		}
		else
			newWaveHeader->dwUser=0;
		
		err_q=QueueWaveBuf(newWaveHeader);
	//	delete newWaveHeader;
		if (err_q != 10 || recStopped==1)
			break;
	}
   
    if (err_q == 10){ //playStopped
		WaitForSingleObject(m_hCleanThread, INFINITE);
		ret_val = 0;
	}
	else //Error
		ret_val= -1;

	//Stop Windows queuing of buffers (and calling of my callback) 
	waveInReset(HWaveIn);

    //Close WAVE Out device 
	waveInClose(HWaveIn);

    return ret_val; 
}


//---------------------------------------------------
// This routine run in a sperate thread, takes care of 
// the garbage handling of the returned (playbacked) waveheaders
// from the waveIn device
//--------------------------------------------------- 
int WavRec::CleanWave()
{
	MSG msg;
	short done=0;
	int err;

	while (GetMessage(&msg, NULL, 0, 0)) 
	{
//		printf("rec clean wave\n");
		if (msg.message==WM_USER+502)
		{   
			LPWAVEHDR waveHeader= (LPWAVEHDR)msg.lParam;
			err=ReleaseData(waveHeader);
			if (err <=0){  //Socket failure
				recStopped=1; //Sign for play_stop
				if (parent_window != NULL) //Notifies this the parent window
					PostMessage(parent_window, WM_USER+11, NULL,NULL);
				
			}
			
			waveInUnprepareHeader( HWaveIn, waveHeader /*(LPWAVEHDR)msg.lParam*/, sizeof(WAVEHDR) );
			nQueued--;
			if (((LPWAVEHDR)msg.lParam)->dwUser == 1)
				done=1;
			else if (nQueued==QueueSize) SetEvent(Event);        
			delete (void*)msg.lParam;
		}
		else if (msg.message==WM_QUIT)
		{
			return 0;
		}
		TranslateMessage(&msg);
		DispatchMessage(&msg);

		if (done ==1)
			break;
	}
	return 0;
}


//-------------------------------------------
//Interface to the external world, which starts the 
// playing thread
//----------------------------------------------
void WavRec::startRec()
{
	//We make a fresh start
	//pl_state=0;
	recStopped=0;
	recStopped_forced=0;

    m_hRecThread = CreateThread(NULL,
								0,
								&recThread,
								(LPVOID) this,
								// CREATE_SUSPENDED,
								0,
								NULL);
#ifndef _WIN32_WCE
//	WaitForSingleObject(m_hPlayThread, INFINITE);
#endif
//	printf("thread exits");

}




//----------------------------------------------
//Stops the playing thread
//----------------------------------------------
void WavRec::stopRec(){
	recStopped_forced=1;
   

}


//-----------------------------------------
//Waits until the record thread has completed
//----------------------------------------- 
void WavRec::waitRec(){
  WaitForSingleObject(m_hRecThread, INFINITE);
}


//------------------------------------------
// Adds a sync object
//---------------------------------------------
void WavRec::addSync(Sync *s){
	sync_obj=s;
}

//------------------------------------------
// Sets the verbosity
//--------------------------------------
void WavRec::setVerbosity(int v){
	verbosed=v;
}

//---------------------------------------
// Sets the common logfile for play and record
//--------------------------------------
void WavRec::setLogFD(FILE *fd){
	VoipLog_fd= fd;
}


//------------------------------------------
// Sets the microphone off
//------------------------------------------
void WavRec::set_micOff(){
	mic_off =1;
}

//----------------------------------------
// Sets the microphone on
//----------------------------------------
void WavRec::set_micOn(){
	mic_off =0;
}

//-------------------------------------------
// Callback function for the waveIn device
//-------------------------------------------
void  CALLBACK WaveInProc(HWAVEIN waveIn, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2)
{
//	cerr << "Global waveInProc called\n";
//	printf("glb waveInProc called \n");
	WavRec *wrecobj =(WavRec *) dwInstance;
	PostThreadMessage(wrecobj->id_CleanThread,WM_USER+502,NULL,dwParam1);

}

//-------------------------------------
// Auxliary thread caller
//-----------------------------------
DWORD WINAPI recThread(VOID *vptr)
{
	WavRec *wrecobj = (WavRec*)vptr;
    if (wrecobj->RecWave() <0)
		exit(0); 

	return 1;
}

//--------------------------------------
// Auxiliary thread caller
//-----------------------------------
DWORD WINAPI RecCleanThread(VOID *vptr)
{
	((WavRec*)vptr)->CleanWave();
	return 1;
}




#endif


/* ******************************** main() ******************************** */

#ifdef __STANDALONE

int main(int argc, char **argv)
{
//	if (argc != 3){
//		cerr << "WavRec plays 8/16Khz, 16 bit PCM files\n";
//		cerr << "USAGE : WavRec pcmfilename  8000/16000\n";          
//		exit(0);
//	}

socket_stream *ss = new socket_stream("127.0.0.1");
WavRec	*wrobj = new WavRec(ss,NULL);//"148.121.142.16");
WavPly  *wpobj =new WavPly(ss,NULL);
	//wpobj->SetSampRate(atoi(argv[2]));
	//wpobj->PlayWave(argv[1]);
    wrobj->startRec();
	Sleep(10000);
	wrobj->stopRec();
//printf("__---___----_____start playing");
wpobj->startPlay();
Sleep(10000);
 wpobj->stopPlay();

	return(-1);
}

#endif